import os
import sys
from enum import Enum
from importlib.util import find_spec
from pathlib import Path
import subprocess

from git import Repo, GitCommandError
from rich.prompt import Prompt
from typer import Typer

from mydev.console import console
from mydev.constants import MYDEV_GIT_INSTALL_DIR, MYDEV_REPO_SSH_URL, MYDEV_REPO_HTTPS_URL

infra_app = Typer(help="Infrastructure management")


@infra_app.command(name="shell")
def shell():
    pass


# https://github.com/tiangolo/typer/pull/669
class Protocol(str, Enum):
    ssh = "ssh"
    https = "https"


@infra_app.command(name="mydev")
def mydev(install_dir: str = MYDEV_GIT_INSTALL_DIR, protocol: Protocol = Protocol.ssh):
    def _clone_via_ssh():
        console.print("Attempting to clone via ssh...")
        Repo.clone_from(MYDEV_REPO_SSH_URL, install_dir)

    def _clone_via_https():
        github_token = Prompt.ask("Enter GitHub token", password=True)
        repo_url = MYDEV_REPO_HTTPS_URL.replace("https://", f"https://{github_token}:x-oauth-basic@")
        console.print("Attempting to clone via https...")
        Repo.clone_from(repo_url, install_dir)

    # clone mydev
    if os.path.exists(install_dir):
        console.print("MyDev local repository found")
        # pull latest changes if the repo is not up-to-date
        repo = Repo(install_dir)
        try:
            repo.remotes.origin.fetch()
            if repo.head.commit != repo.remotes.origin.refs.main.commit:
                repo.remotes.origin.pull()
                console.print("Pulled successfully")
            else:
                console.print("The repository is up-to-date")
        except GitCommandError as e:
            console.print(f"Failed to pull: {e}")
    else:
        os.makedirs(os.path.dirname(install_dir), exist_ok=True)
        try:
            if protocol == "ssh":
                _clone_via_ssh()
            else:
                _clone_via_https()
            console.print("Cloned successfully")
        except GitCommandError as e:
            console.print(f"Failed to clone via {protocol}: {e}")
            try:
                if protocol == "ssh":
                    _clone_via_https()
                else:
                    _clone_via_ssh()
                console.print("Cloned successfully")
            except GitCommandError as e:
                console.print(f"Failed to clone via {'ssh' if protocol == 'https' else 'https'}: {e}")

    # install mydev in editable mode
    if not Path(find_spec("mydev").origin).is_relative_to(Path(install_dir)):
        console.print("Installing MyDev in editable mode...")
        subprocess.check_call([sys.executable, "-m", "ensurepip"])
        subprocess.check_call([sys.executable, "-m", "pip", "install", "-e", install_dir])
    else:
        console.print("MyDev is already installed in editable mode")


@infra_app.callback(invoke_without_command=True, no_args_is_help=True)
def main():
    pass
